/*****************************************************************************
*
*	File Name:  lcd.c                                         
*                                                                  
*	Content:  LCD routines for 4x16 character display from GPO                
*                                                                  
*  	Date:  21-08-02
*                                                                  
*	Copyright 2002
*	by R.Inacker
*                                                                  
*=============================================================================
*	$Log:$
*===========================================================================*/
//#include "Config.h"
#include <inttypes.h>
#include <avr/io.h>
#include <avr/delay.h>
#include "lcd.h"


//volatile int timeout=0;

/*****************************************************************************
Name:    delay_1ms       
Parameters:                     
Returns:        
Description:   Routine that creates a 1ms delay 

*****************************************************************************/

void delay_1ms(void)                 
{
//wait (10000 x 4) cycles = wait 40000 cycles
		_delay_loop_2(10000);
}

/*****************************************************************************
Name:    delay_xms    delay mit x-millisecound       
Parameters:           max. 131 ms          
Returns:        
Description:   Routine that creates a 1ms delay 

*****************************************************************************/

void delay_xms(long time)                 
{
  while (time !=0)
		{
		//wait (10000 x 4)cycles = wait 40000 cycles
		_delay_loop_2(10000);
		time--;
		}
}

void delay_xs(double second)                 
{        
        second=second*10;  
        while (second !=0)
                {delay_xms(100);
                 second--;
                }
}



/*****************************************************************************
Name:   LCD_init        
Parameters: none                    
Returns:   none     
Description:  LCD initialization routine

*****************************************************************************/

void LCD_init(void) 
{
		delay_xms(20);
unsigned short i;
	lcd_control_dir =0xff;
   	for (i=15; i>0; i--)
   		delay_1ms();  		/* startup delay 15mS */
   			
        lcd_out_dir=0xff;
        lcd_control = 0x00;             /*lsche RS + R/W */
        lcd_out= disp_mode;       
        toggle_enable();

        delay_xms(4);                   /* 4ms*/
        lcd_out= disp_mode;
        toggle_enable();
        delay_1ms();                    /* 100us*/
        lcd_out= disp_mode;
        toggle_enable();
        lcd_out= disp_mode;             /* set interface length and line*/
        toggle_enable();
        lcd_out= disp_off;              /* display off*/
        toggle_enable();
        lcd_out= clr_dsp;               /* clear display */
        toggle_enable();
        lcd_out= inc_entry;             /* display off*/
        toggle_enable();
/* ********busy is now avaible ***************/

	delay_1ms();
	SendCmd(disp_mode);		/* set display mode */
	delay_1ms();
	SendCmd(disp_mode);		/* set display mode */
	delay_1ms();
	SendCmd(disp_mode);		/* set display mode */
	delay_1ms();
	SendCmd(inc_entry);		/* set entry mode, increment on */
	delay_1ms();
    SendCmd(disp_on);		/* display on */
	delay_1ms();
	clrLCD();			/* clear LCD */
}

/******************************************************************
	SendChar - Sends character to LCD                               
******************************************************************/
void Send_Char(char lcd_data)
{
    //busycheck();
	lcd_out_dir |= 0xFF;
	lcd_out = lcd_data;	        /* put data on Port 0 */
	set_rs;						/* Set rs for data */
	clr_read_write;				/* write data */
	toggle_enable ();
	delay_1ms();
}

/******************************************************************
	SendCmd - Sends command to LCD
******************************************************************/
void SendCmd(uint8_t lcd_data)
{
	//busycheck();
	lcd_out_dir |= 0xFF;	
    lcd_out = lcd_data;	        /* put data on Port 0 */
	clr_rs;	 					/* clear for command */
	clr_read_write;   			/* clear for command */
	toggle_enable ();
	delay_xms(1);
	
}

/******************************************************************
	clrLCD - Clear the contents of the LCD
******************************************************************/
void clrLCD(void)
{
	SendCmd(clr_dsp);		/* clear display */
	delay_1ms();
}

/******************************************************************
	L1homeLCD - Moves the cursor to home position on Line 1         
******************************************************************/
void L1homeLCD(void)
{
	SendCmd(DD_RAM_AD10);	/* home position on line 1 */
	delay_1ms();
}

/******************************************************************
	L2homeLCD - Moves the cursor to home position on Line 2 
******************************************************************/
void L2homeLCD(void)
{
	SendCmd(DD_RAM_AD20);	/* home position on line 2 */
	//delay_1ms();
}

/******************************************************************
	L3homeLCD - Moves the cursor to home position on Line 3         
******************************************************************/
void L3homeLCD(void)
{
	SendCmd(DD_RAM_AD30);	/* home position on line 3 */
	delay_1ms();
}

/******************************************************************
	L4homeLCD - Moves the cursor to home position on Line 4         
******************************************************************/
void L4homeLCD(void)
{
	SendCmd(DD_RAM_AD40);	/* home position on line 1 */
	delay_1ms();
}

/******************************************************************
	L1endLCD - Moves the cursor to end of Line 1         
******************************************************************/
void L1endLCD(void)
{
	SendCmd(DD_RAM_AD1E);	/* last address of line 1 */
	delay_1ms();
}

/******************************************************************
	L2endLCD - Moves the cursor to end of Line 2         
******************************************************************/
void L2endLCD(void)
{
	SendCmd(DD_RAM_AD2E);	/* last address of line 2 */
	delay_1ms();
}

/******************************************************************
	L3endLCD - Moves the cursor to end of Line 3         
******************************************************************/
void L3endLCD(void)
{
	SendCmd(DD_RAM_AD3E);	/* last address of line 3 */
	delay_1ms();
}

/******************************************************************
	L4endLCD - Moves the cursor to end of Line 4         
******************************************************************/
void L4endLCD(void)
{
	SendCmd(DD_RAM_AD4E);	  /* last address of line 4 */
	delay_1ms();
}

/******************************************************************
	LCD_entry - Puts the LCD in normal increment entry mode 
******************************************************************/
void LCDinc(void) 
{
	SendCmd(inc_entry);	  /* set entry mode, increment on */
	delay_1ms();
}

/******************************************************************
	LCD_scroll - Puts the LCD in scroll entry mode 
******************************************************************/
void LCDscr(void)
{
	SendCmd(scr_entry);
	delay_1ms();
}

/******************************************************************
	set enable-signal 
******************************************************************/
void toggle_enable(void)
{
    set_enable;				//lcd_enable = 1
    asm volatile ("NOP");	// setup time           
    asm volatile ("NOP");	// setup time           
	clr_enable;				//lcd_enable = 0;lcd_enable = 0;
}

void busycheck(void)
{
		volatile int c=1;
		lcd_out_dir &= 0x00;          /* Datenport auf Eingang*/
        clr_rs;                     /* setzt D/I            */
        set_read_write;			    //rs =1;                      /* setzt R/W            */	
		toggle_enable ();
		while (c){
		c = inp(lcd_out) & (1<<busy);
		delay_xms(1);
		}
	    clr_enable;
		lcd_out_dir |= 0xFF;
}

void SetCursor(uint8_t lcd_data)
{
    lcd_data =lcd_data + 0x80;
	SendCmd(lcd_data);
	delay_1ms();
}

void LCD_write(uint8_t *msg_string, int line )
{
	int i;

	switch(line){
		case 1:
                        L1homeLCD();			    // home display on line 1
                        break;
        	case 2:
                        L2homeLCD();			    // home display on line 2
                        break;
	        case 3:
                        L3homeLCD();			    // home display on line 3
                        break;
	        case 4:
                        L4homeLCD();			    // home display on line 4
                        break;
                default:
                     L1homeLCD();			    	// home display on line 1
            }
	for (i=0;  msg_string[i] ;i++) {	// This loop reads in the text string 
			Send_Char(msg_string[i]);
             }

}

void lcd_puts(const char *s)
/* print string on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = *s++) ) {
        Send_Char(c);
    }

}/* lcd_puts */




