/*****************************************************************************
*
*	File Name:  lcd.h                                         
*                                                                  
*	Content:  LCD header 4 x 16 LCD module von GPO
*
*  	Date:  26-07-02
*                                                                  
*	                         
*	All rights reserved                                            
*                                                                  
*=============================================================================
*	$Log:$
*===========================================================================*/


/********************************************************
	declare LCD connections				
********************************************************/
#ifndef __LCD_H
#define __LCD_H

#include <inttypes.h>

#define		rs				PORTD.0  	/* LCD register select */
#define     set_rs          PORTD |= 1<<0;
#define     clr_rs          PORTD &= 0xFE;
#define		rw				PORTD.1   	/* LCD read/write pin */
#define     set_read_write  PORTD |= 1<<1;
#define     clr_read_write  PORTD &= 0xFD;
#define		lcd_enable 		PORTD.2     /*LCD enable pin */
#define     set_enable      PORTD |= 1<<2;
#define     clr_enable      PORTD &= 0xFB;
#define		lcd_control		PORTD   	/* LCD control output */
#define		lcd_control_dir	DDRD   		/* LCD control output direction*/
#define		lcd_out			PORTA   	/* LCD data output */
#define     lcd_out_dir 	DDRA        /* LCD data port direction*/
#define		busy  			7		   	/* busy flag */
/********************************************************
	declare LCD instructions				
********************************************************/

#define		clr_dsp	 	0x01	/* clear display */
#define 	disp_on	 	0x0C	/* Display on */
#define 	disp_c_on	0x0E	/* Display on, Cursor on */
#define 	disp_b_on	0x0F	/* Display on, Cursor on, Blink cursor */
#define 	disp_off 	0x08	/* Display off */
#define		disp_mode	0x38	/* 8 bit, 2 line, 5x7 mode */
#define		inc_entry	0x06	/* increment entry mode */
#define		scr_entry	0x05	/* scroll entry mode */	
#define 	DD_RAM_AD10	0x80	/* Line 1 home */		
#define		DD_RAM_AD20	0xC0	/* Line 2 home */
#define 	DD_RAM_AD30	0x94	/* Line 3 home */		
#define		DD_RAM_AD40	0xD4	/* Line 4 home */
#define 	DD_RAM_AD1E	0x93	/* End of line 1 */
#define 	DD_RAM_AD2E	0xD3	/* End of line 2 */
#define 	DD_RAM_AD3E	0xA7	/* End of line 3 */
#define 	DD_RAM_AD4E	0xE7	/* End of line 4 */


/********************************************************
	function prototypes				
********************************************************/
 
extern void delay_1ms(void);
extern void delay_xms(long time);
extern void delay_xs(double second);                   
extern void LCD_init(void);
extern void Send_Char(char lcd_data);
extern void SendCmd(uint8_t lcd_data);
extern void clrLCD(void);
extern void L1homeLCD(void);
extern void L2homeLCD(void);
extern void L3homeLCD(void);
extern void L4homeLCD(void);
extern void L1endLCD(void);
extern void L2endLCD(void);
extern void L3endLCD(void);
extern void L4endLCD(void);
extern void LCDinc(void);
extern void LCDscr(void);
extern void toggle_enable(void);
extern void busycheck(void);
extern void SetCursor(uint8_t lcd_data);
extern void LCD_write(uint8_t *msg_string, int line);
extern void lcd_puts(const char *s);
#endif
